/**

 --- WS281x light controller ---
 ATtiny85 based
 Version 1.1 - 14 December 2023

Button 1 is connected between PB2 and ground
Button 2 is connected between PB1 and ground
LED strip data line is connected to PB0
Microphone (signal, reference) is connected to PB4, PB3. ADC is used in differential mode.

Fuse values: H=0xDD, L=0xF1, E=0xFF

Button 1 is used to switch mode. Button 2 is used to switch brightness.
    - Buttons act on release.
    - Holding down both buttons for 2 seconds and releasing after the strip blinks saves current mode and brightness into EEPROM

To enter setup mode, hold both buttons before powerup, then turn on power with both buttons held
    - After 5 seconds, the strip will blink once.
    - Then release press both switches rapidly (>16 state changes in 5 seconds). The strip will blink.
    - Then, RGB/GRB mode (LED type) can be selected.
        - A RGB or GRB pattern appears on the first 3 LEDs.
        - Change the pattern using button 1 (A 3 LED "RGB" pattern should appear on the LED strip)
            - WS2812B and similar LEDs are addressed as GRB, some chains as RGB.
        - Press button 2 to save.
    - The LED strip should flash again, release both buttons
    - After a second, a single LED (originally at position 20) will be lit. Then the LED count can be set.
        - Each press of button 1 moves the lit LED position (from 20 up to 60, then back to 5 and up again).
        - This sets total LED count (up to 60).
        - Press button 2 to save.
    - After a second, two LEDs will stay lit - first one normal white, fourth one "new" white. Then the white tint can be set.
        - Each press of button 1 moves changes the tint. 9 options total.
            - This sets the LED tint according to the following sequence (R, G, B), H=160, M=120, L=80
            (M, M, M) -> (H, M, L) -> (L, M, H) -> (M, L, M) -> (H, L, L) -> (L, L, H) -> (M, H, M) -> (H, H, L) -> (L, H, H)
    - Pressing button 2 saves the settings into EEPROM and the device enters normal operation.

Modes:
    - Autoranging VU meter, green/red mode
    - Autoranging VU meter, rainbow mode, starting with red color
    - Autoranging VU meter, rainbow mode, with fast moving color
    - White light
    - Red light
    - Green light
    - Blue light
    - Fast moving rainbow
    - Slow moving rainbow
    - Slow moving triple rainbow
    - Fast moving shimmering
    - Slow moving shimmering


Avrdude command:
avrdude -c usbasp -p t85 -B10 -U hfuse:w:0xdd:m -U lfuse:w:0xf1:m -U efuse:w:0xff:m -U flash:w:'/path/to.hex'

This project is licensed under the 3-clause BSD licence:

Copyright 2021, 2023 Frantisek Stefanec

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/eeprom.h>
#include "ws2812b.h"
#include "config.h"

#define FAST_MOVING_STEP 300u // ~72 degree color wheel diff per second
#define SLOW_MOVING_STEP 20u // ~5 degree color wheel diff per second

#define SW1_PIN PB2
#define SW2_PIN PB1

// default type & count
uint8_t led_type = 0, led_count = 20;

// approximate exponential function
const uint8_t brightness_table[8] = {16, 24, 35, 52, 78, 116, 172, 255};

// White tint setting
const uint8_t led_tint_r_arr[3] = {120, 170, 80};
const uint8_t led_tint_g_arr[3] = {120, 150, 90};
const uint8_t led_tint_b_arr[3] = {120, 80, 160};
uint8_t led_tint = 0; // 9 choices
uint8_t led_tint_r, led_tint_g, led_tint_b;

uint8_t brightness = 0, brightness_selected=0, mode = 0;
uint16_t angle2 = 0; // color wheel angle difference per LED with max 255*256

uint16_t angle_vu = 0; // start for rainbow VU meter

// shimmer effect variables and defines
uint16_t shimmer_timing = 0;
uint8_t shimmer_led_states[16], shimmer_prediv_ctr;

#define SHIMMER_STEPS 255 // 63, 127 or 255
#define SHIMMER_BRI_INT ((uint8_t)(255/SHIMMER_STEPS))

uint8_t stat = 0; // status - timer triggered etc

// variables for peak detection, VU meter function etc
uint16_t norm_min=65535, norm_max=0;
uint16_t norm_val;
uint16_t tmp_read;
uint16_t tmp_read_array[12];
uint8_t tmp_read_array_pos = 0;
uint8_t measure_ctr;
uint16_t peak_leave_alone_counter, val_leave_alone_counter;
uint32_t peak_val;
uint8_t sound_disp_value;
uint16_t peak_adjust_prediv_counter;
uint8_t old_adc_data; // for rng improvement

// for rainbow effect, color wheel angle, 0-255 upscaled by 256
uint16_t angle = 0;

// random number generator variables and defines
uint16_t xorshift_internal = 0;
#define XORSHIFT_SEED 0x6789

// timer interrupt
ISR(TIMER1_COMPA_vect)
{
    stat |= 1;
}

uint8_t ctr=0;
uint8_t wait_for_sw_enable=0;

// Color wheel and brightness functions
inline uint8_t wheel_r(uint8_t angle) {
    return angle < 85 ? (255 - angle*3) : (angle > 170 ? 3 * (angle-170) : 0);
}

inline uint8_t wheel_g(uint8_t angle) {
    return angle < 170 ? (angle < 85 ? 3*(angle-85) : (3 * (170 - angle))) : 0;
}

inline uint8_t wheel_b(uint8_t angle) {
    return angle >= 85 ? (angle < 170 ? 3*(angle-85) : (3 * (255 - angle))) : 0;
}

// brightness function
inline uint8_t bri (uint8_t value, uint8_t brightness) {
    // for unknown reason it doesn't work with uint16_t
    // fixing it/finding out why would optimize code size and speed
    if (value==0 || brightness==0) return 0;
    uint32_t x=value;
    x *= brightness;

    return (uint8_t)(( (255u+(uint16_t)x) & 0xff00)>>8);
}

// set ADC gain function
inline void set_admux_gain(uint8_t gain_sel) { // gain_sel=0 -> gain=1, else gain=20
    ADMUX = (1<<REFS1) | 0b0110 | (gain_sel ? 0b0001 : 0);
}

// button related
uint8_t sw1_down_time=0, sw2_down_time=0;
uint8_t sw_status=0, prev_sw_status=0; // 1: sw1, 2: sw2

void refresh_sw_status() { // using global variables
    prev_sw_status = sw_status;

    if (((~PINB) & (1<<SW1_PIN)) && sw1_down_time<4) sw1_down_time++;
    else if (sw1_down_time > 0) sw1_down_time--;

    if (((~PINB) & (1<<SW2_PIN)) && sw2_down_time<4) sw2_down_time++;
    else if (sw2_down_time > 0) sw2_down_time--;

    if (sw1_down_time > 2) sw_status |= 1;
    else if (sw1_down_time < 2) sw_status &= ~(1);

    if (sw2_down_time > 2) sw_status |= 2;
    else if (sw2_down_time < 2) sw_status &= ~(2);
}

// xorshift with option of flipping random bit (lsb of xorshift_insert_data)
uint8_t xorshift_random(uint8_t xorshift_insert_data) {
    if (xorshift_internal == 0) xorshift_internal = XORSHIFT_SEED;
    if ((xorshift_insert_data & 1) != 0 && ((xorshift_internal ^ (((uint16_t)xorshift_insert_data) & 1u)) != 0)) xorshift_internal ^= ((uint16_t)xorshift_insert_data);
    // xorshift code by John Metcalf & G. Marsaglia
    xorshift_internal ^= xorshift_internal << 7;
    xorshift_internal ^= xorshift_internal >> 9;
    xorshift_internal ^= xorshift_internal << 8;

    return (uint8_t)(xorshift_internal & 0xff);
}

// user setup, led count etc
void setup() {
    if ((PINB & 6) != 0) return; // Both buttons down? If not, return

    uint16_t dual_press_counter=0, i=0;
    while ((PINB & 6) == 0 && dual_press_counter < 500) {
        _delay_ms(10); // at least 5 second press to enter
        dual_press_counter++;
    }
    if (dual_press_counter<500) return;

    // Blink all to confirm long dual press
    for (i=0; i<60; i++) set_led_color(i,30,30,30,0);
    led_refresh(60);
    i=20;
    while(i--) _delay_ms(10);
    for (i=0; i<60; i++) set_led_color(i,0,0,0,0);
    led_refresh(60);

    uint8_t btn = 6, prev_btn = 6, changes = 0;
    i=500;
    while (i--) {
        prev_btn = btn;
        btn = (PINB & 6);
        if (btn != prev_btn) changes++;
        _delay_ms(10);
    }
    if (changes < 16) return; // at least 16 button changes in 5 sec to enter setup

    // Long blink all to confirm entering setup
    for (i=0; i<60; i++) set_led_color(i,30,30,30,0);
    led_refresh(60);
    i=100;
    while(i--) _delay_ms(10);
    for (i=0; i<60; i++) set_led_color(i,0,0,0,0);
    led_refresh(60);

    /// first setup GRB or RGB mode
    uint8_t in_loop = 1;
    while (in_loop) {
        if (~PINB & (1<<SW1_PIN)) {
            led_type++; // SW1 = switch LED type
            if (led_type > 1) led_type = 0;
        } else if (~PINB & (1<<SW2_PIN)) {
            in_loop = 0; // SW2 = "save" (exit loop)
        }

        // first 3 leds = RGB
        set_led_color(0, 255, 0, 0, led_type);
        set_led_color(1, 0, 255, 0, led_type);
        set_led_color(2, 0, 0, 255, led_type);
        led_refresh(3);

        while(~PINB & (1<<SW1_PIN)) _delay_ms(10);
        _delay_ms(10);
    }

    // leds off
    for (i=0; i<60; i++) set_led_color(i,0,0,0,0);
    led_refresh(60);

    /// Setup LED count
    // 1s delay
    i=100;
    while(i--) _delay_ms(10);

    in_loop = 1;
    uint8_t prev_led_count=led_count;

    while (in_loop) {
        prev_led_count=led_count;
        if (~PINB & (1<<SW1_PIN)) {
            led_count++; // SW1 = increase led count
            if (led_count > 60) led_count = 5;
        } else if (~PINB & (1<<SW2_PIN)) {
            in_loop = 0; // SW2 = "save" (exit loop)
        }

        // last LED= white
        set_led_color(prev_led_count-1, 0, 0, 0, led_type);
        set_led_color(led_count-1, 85, 85, 85, led_type);
        led_refresh(60);
        while(~PINB & (1<<SW1_PIN)) _delay_ms(10);
        _delay_ms(10);
    }

    // leds off
    for (i=0; i<60; i++) set_led_color(i,0,0,0,0);
    led_refresh(60);

    /// Set white LED tint (temperature)
    i=100;
    while(i--) _delay_ms(10);

    in_loop = 1;

    while (in_loop) {
        if (~PINB & (1<<SW1_PIN)) {
            led_tint++; // change led tint
            if (led_tint > 8) led_tint = 0;
        } else if (~PINB & (1<<SW2_PIN)) {
            in_loop = 0; // SW2 = "save" (exit loop)
        }

        // first led = normal white, second led = adjusted white
        set_led_color(0, 120, 120, 120, led_type);
        set_led_color(3, led_tint_r_arr[led_tint%3], led_tint_g_arr[led_tint/3], led_tint_b_arr[led_tint%3], led_type);
        led_refresh(60);
        while(~PINB & (1<<SW1_PIN)) _delay_ms(10);
        _delay_ms(10);
    }


    // 1s delay
    i=100;
    while(i--) _delay_ms(10);

    // save and short blink all leds
    eeprom_write_byte((uint8_t*)10, led_type);
    eeprom_write_byte((uint8_t*)11, led_count);
    eeprom_write_byte((uint8_t*)12, led_tint);

    for (i=0; i<led_count; i++) set_led_color(i,30,30,30,0);
    led_refresh(led_count);
    i=20;
    while(i--) _delay_ms(10);
    for (i=0; i<led_count; i++) set_led_color(i,0,0,0,0);
    led_refresh(led_count);
    i=50;
    while(i--) _delay_ms(10);

}

int main(void) {
    DDRB = (1<<PB0); // Output for LEDs
    PORTB = (1<<PB1) | (1<<PB2); // Button pull ups
    led_clear(led_count);

    // setup adc
    set_admux_gain(1);
    ADCSRB |= (1<<BIN); // bipolar mode
    ADCSRA = (1<<ADPS1) | (1<<ADPS2) | (1<<ADEN) | (1<<ADSC); // cca 19.2 ksps
    while ((ADCSRA & (1<<ADSC)) != 0);

    // setup timer (62 Hz refresh)
    TCCR1 = (1<<CTC1) | 0b1011; // :1024
    OCR1A = OCR1A_VAL;
    TIMSK |= (1<<OCIE1A);

    // clear all leds
    uint8_t l=0;
    for (l=0; l<60; l++) set_led_color(l,0,0,0,0);
    led_refresh(60);

    // delay and enter setup if buttons pressed
    _delay_ms(10);
    setup();

    // set color wheel angle per LED
    angle2 = 65280u/led_count;

    // generate random numbers for shimmer
    for(uint8_t k=0; k<16; k++) shimmer_led_states[k] = xorshift_random(0);

    // load mode and brightness from EEPROM
    mode = eeprom_read_byte((uint8_t*)0);
    brightness_selected = eeprom_read_byte((uint8_t*)1);
    led_type = eeprom_read_byte((uint8_t*)10);
    led_count = eeprom_read_byte((uint8_t*)11);
    led_tint = eeprom_read_byte((uint8_t*)12);

    // check if values correct
    if (mode>11) mode=0;
    if (brightness_selected>7) brightness_selected=0;
    if (led_type>1) led_type=0;
    if (led_count>60) led_count=20;
        else if (led_count < 5) led_count = 5;
    if (led_tint>8) led_tint=0;
    led_tint_r = led_tint_r_arr[led_tint % 3];
    led_tint_g = led_tint_g_arr[led_tint / 3];
    led_tint_b = led_tint_b_arr[led_tint % 3];

    // main loop
    while(1)
    {
        // ADC controller: do a conversion:
        ADCSRA |= (1<<ADSC);
        while ((ADCSRA & (1<<ADSC)) != 0);
        tmp_read = ADC;
        tmp_read = (uint16_t)(512+(int16_t)(tmp_read | ((tmp_read & (1<<9)) ? (0b11111100<<8) : 0))); // polarity setting etc
        old_adc_data = (uint8_t)tmp_read;

        // primitive filtering
        tmp_read_array[tmp_read_array_pos] = tmp_read;
        tmp_read_array_pos++;
        if (tmp_read_array_pos == 12) {
            uint16_t temp_sort;
            // get medians of every 3 measured points filter
            for(uint8_t k=0; k<4; k++) {
                if (tmp_read_array[0+k*3] > tmp_read_array[1+k*3]) {
                    temp_sort = tmp_read_array[0+k*3];
                    tmp_read_array[0+k*3] = tmp_read_array[1+k*3];
                    tmp_read_array[1+k*3] = temp_sort;
                }
                if (tmp_read_array[1+k*3] > tmp_read_array[2+k*3]) {
                    temp_sort = tmp_read_array[1+k*3];
                    tmp_read_array[1+k*3] = tmp_read_array[2+k*3];
                    tmp_read_array[2+k*3] = temp_sort;
                }
                if (tmp_read_array[0+k*3] > tmp_read_array[1+k*3]) {
                    temp_sort = tmp_read_array[0+k*3];
                    tmp_read_array[0+k*3] = tmp_read_array[1+k*3];
                    tmp_read_array[1+k*3] = temp_sort;
                }
            }

            uint16_t filtered_val = tmp_read_array[1] + tmp_read_array[4] + tmp_read_array[7] + tmp_read_array[10];

            if (filtered_val <= norm_min) norm_min = filtered_val;
            if (filtered_val >= norm_max) norm_max = filtered_val;
            tmp_read_array_pos=0;
        }

        if ((stat & 1) != 0) // if timer trig'd
        {
            stat &= ~1;
            measure_ctr++; // "prediv", do some things on every second timer interrupt
            if (measure_ctr >= 2) measure_ctr = 0;

            if (measure_ctr==0)// get maximum, minimum and difference
            {
                norm_val = norm_max-norm_min;
                norm_val = norm_val >= NOISE_REDUCTION ? norm_val-NOISE_REDUCTION : 0;
                norm_min = 65535;
                norm_max = 0;

            }

            if (norm_val > peak_val) // save peak
            {
                peak_leave_alone_counter = PEAK_LEAVE_ALONE_TIME;
                peak_val = norm_val-(norm_val/32);
            }

            if (peak_leave_alone_counter > 0) peak_leave_alone_counter--; // wait
            else // if time reached, decrease peak
            {
                uint16_t pk_decrease = (peak_val/PEAK_DECREASE_STEPDIV);
                if (pk_decrease == 0) pk_decrease = 1;

                if (peak_val > pk_decrease)
                {
                    peak_val -= pk_decrease;
                }
                if (peak_val < MIN_PEAK_VALUE) peak_val = MIN_PEAK_VALUE;
            }

            uint8_t sound_disp_hit = (uint8_t) (((unsigned long)led_count*norm_val)/((unsigned long)peak_val));

            if (sound_disp_hit >= sound_disp_value)
            {
                sound_disp_value = sound_disp_hit;
                val_leave_alone_counter = VAL_LEAVE_ALONE_TIME;
            }

            if (val_leave_alone_counter > 0) val_leave_alone_counter--;
            else
            {
                uint8_t dec = (sound_disp_value/VAL_DECREASE_STEPDIV);
                if (dec==0) dec=1;
                if (sound_disp_value > dec) sound_disp_value -= dec;
                else sound_disp_value = 1;

            }



            if (sound_disp_value < 1) sound_disp_value = 1;

            if (sound_disp_value > led_count) sound_disp_value = led_count;

            // read brightness from table
            brightness = brightness_table[brightness_selected];

            // temporary RGB, universal
            uint8_t tr, tg, tb, tu;

            uint16_t angle_per_led_r_vu; // total color wheel angle shift per LED
            switch (mode) { // VU meter mode
                case 0:
                    for (uint8_t i=0; i<led_count; i++)
                    {
                        if (sound_disp_value > i)
                        {
                            uint8_t tu = bri(255,brightness);
                            if (sound_disp_value > i+2) set_led_color(i, 0, brightness, 0, led_type);
                            else if (sound_disp_value > i+1) set_led_color(i, tu, tu/2, 0, led_type);
                            else set_led_color(i, brightness, 0, 0, led_type);
                        }
                        else set_led_color(i, 0, 0, 0, led_type);
                    }
                    break;

                case 1: // rainbow VU meter mode
                case 2: // rainbow VU meter mode with color rotation
                    angle_per_led_r_vu = 65280u/(uint16_t)sound_disp_value;
                    if (mode==2) angle += FAST_MOVING_STEP;

                    for (uint8_t i=0; i<led_count; i++)
                    {
                        if (sound_disp_value > i)
                        {
                            uint8_t color_wheel_angle;
                            if (mode == 1) color_wheel_angle = (uint8_t)((((uint32_t)angle_vu + i*angle_per_led_r_vu) >> 8) & 0xff);
                            else color_wheel_angle = (uint8_t)((((uint32_t)angle + i*angle_per_led_r_vu) >> 8) & 0xff);
                            tu=wheel_r(color_wheel_angle);
                            tr=bri(tu, brightness);
                            tu=wheel_g(color_wheel_angle);
                            tg=bri(tu, brightness);
                            tu=wheel_b(color_wheel_angle);
                            tb=bri(tu, brightness);
                            set_led_color(i, tr, tg, tb, led_type);
                        }
                        else set_led_color(i, 0, 0, 0, led_type);
                    }
                    break;

                case 3: // white light mode
                    for (uint8_t i=0; i<led_count; i++) {
                        set_led_color(i, bri(led_tint_r, brightness), bri(led_tint_g, brightness), bri(led_tint_b, brightness), led_type);
                    }
                    break;

                case 4:
                case 5:
                case 6: // single color mode
                    for (uint8_t i=0; i<led_count; i++) {
                        set_led_color(i, (mode == 4 ? brightness : 0), (mode == 5 ? brightness : 0), (mode == 6 ? brightness : 0), led_type);
                    }
                    break;

                case 7:
                case 8:
                case 9: // moving colorful

                    for (uint8_t i=0; i<led_count; i++) {
                        uint8_t color_wheel_angle = (uint8_t)((((uint32_t)angle + (mode == 9 ? 3u*i : i)*angle2) >> 8) & 0xff);
                        tu=wheel_r(color_wheel_angle);
                        tr=bri(tu, brightness);
                        tu=wheel_g(color_wheel_angle);
                        tg=bri(tu, brightness);
                        tu=wheel_b(color_wheel_angle);
                        tb=bri(tu, brightness);

                        set_led_color(i, tr, tg, tb, led_type);
                    }
                    angle += (mode == 7) ? FAST_MOVING_STEP : SLOW_MOVING_STEP;

                    break;

                case 10:
                case 11: // shimmering lights mode
                    for (uint8_t i=0; i<led_count; i++) {
                        tu = (shimmer_led_states[i/8] & (1<<(i%8))) ? (SHIMMER_BRI_INT*shimmer_timing) : 0;
                        tu += (shimmer_led_states[8+i/8] & (1<<(i%8))) ? (SHIMMER_STEPS*SHIMMER_BRI_INT)-(SHIMMER_BRI_INT*shimmer_timing) : 0;
                        tu = bri(tu, brightness);
                        tr = bri(led_tint_r, tu);
                        tg = bri(led_tint_g, tu);
                        tb = bri(led_tint_b, tu);
                        // setting minimums for reasonable white color:
                        uint8_t t_max = tr;
                        if (tg > t_max) t_max = tg;
                        if (tb > t_max) t_max = tb;

                        if (t_max < 4) {
                            if (tr < t_max) tr = t_max;
                            if (tg < t_max) tg = t_max;
                            if (tb < t_max) tb = t_max;
                        } else if (t_max < 6) {
                            t_max -= 1;
                            if (tr < t_max) tr = t_max;
                            if (tg < t_max) tg = t_max;
                            if (tb < t_max) tb = t_max;
                        }

                        set_led_color(i, tr, tg, tb, led_type);
                    }

                    shimmer_prediv_ctr++;
                    if (shimmer_prediv_ctr >= ((mode == 10) ? SHIMMER_PREDIV_FASTER : SHIMMER_PREDIV_SLOWER)) {
                        shimmer_timing++;
                        if (shimmer_timing > SHIMMER_STEPS) { // Bitwise ANDs to keep less lights on for power consumption reasons
                            shimmer_led_states[8] = shimmer_led_states[0];
                            shimmer_led_states[0] = xorshift_random(old_adc_data);
                            //shimmer_led_states[0] &= xorshift_random(0);

                            for (uint8_t k=1; k<8; k++) {
                                shimmer_led_states[k+8] = shimmer_led_states[k];
                                shimmer_led_states[k] =  xorshift_random(0);
                                //shimmer_led_states[k] &= xorshift_random(0);

                            }
                            shimmer_timing = 0;
                        }
                        shimmer_prediv_ctr = 0;
                    }
                    break;
                default:break;
            }

            led_refresh(led_count);

            // button handling (press time counter), debounce/hysteresis
            refresh_sw_status();

            if (wait_for_sw_enable>0) wait_for_sw_enable--;

            // button 1, mode switching on release
            if (wait_for_sw_enable==0 && (sw_status & 1)==0 && ((sw_status & 1) != (prev_sw_status & 1))) {
                _delay_ms(1);
                mode++;
            }
            if (mode>11) mode=0;

            // button 2, brightness switching on release
            if (wait_for_sw_enable==0 && (sw_status & 2)==0 && ((sw_status & 2) != (prev_sw_status & 2))) {
                _delay_ms(1);
                brightness_selected++;
            }

            if (brightness_selected>7) brightness_selected=0;

            // save to eeprom  if pressed both buttons for 2+ sec
            if ((PINB & 6) == 0) {
                uint8_t i, save_counter = 0;
                for (i=0; i<60; i++) set_led_color(i,0,0,0,0);
                led_refresh(60);
                while((PINB & 6) == 0 && save_counter < 200) {
                    _delay_ms(10);
                    save_counter++;
                }
                if (save_counter==200) {
                    for (i=0; i<60; i++) set_led_color(i,30,30,30,0); // Blink all to confirm
                    led_refresh(60);
                    i=20;
                    while(i--) _delay_ms(10);
                    for (i=0; i<60; i++) set_led_color(i,0,0,0,0);
                    led_refresh(60);
                    eeprom_write_byte((uint8_t*)0, mode);
                    eeprom_write_byte((uint8_t*)1, brightness_selected);
                }
                while((PINB & (1<<SW1_PIN)) == 0 || (PINB & (1<<SW2_PIN)) == 0) _delay_ms(10);
                i=100;
                while(i--) _delay_ms(10);

                // clear switch variables
                sw_status=0;
                prev_sw_status=0;
                sw1_down_time=0;
                sw2_down_time=0;
                refresh_sw_status();
                wait_for_sw_enable=10; // disable switches for 10 cycles
            }
        }
    }
    return 0;
}
